#define MAIN_BVPT_BUILD_C

#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include "Windows.h"
#include "utils.h"
#include "bvptree.h"
#include "bvtIO.h"

void processArgs(int,char**);		// processes the command-line input arguments

char *datafile;
char *treefile;
char *resultsFile;
int isTreefile=0;
int isResFile = 0;
int n=-1;
int d=-1;
int bucketSize = 50;	// default value for bucketSize

/*global variables for statistics on the tree*/
int maxDepth = 0;	// used for evaluating the maximum depth of the tree
double sumLeafDepth = 0.0;	// each node gives as a contribution the number of data stored in it x its depth
//double NumChildrenOfNonLeaves = 0;
int numNonLeafNodes = 0;
int numLeafNodes = 0;
int constructionCost = 0;

int main(int argc, char** argv){
  
	int i;
	double **x;
	TREENODE root;
  
	LARGE_INTEGER startCount, endCount, freqCount;		// variables used for time measures
	BOOL isHrCounter, isStart, isEnd;												
	isHrCounter = QueryPerformanceFrequency(&freqCount);	// verify the presence of the high-resolution timer

	printf("**** processing input arguments **** \n");
	processArgs(argc,argv);
  
	x = calloc( n,sizeof(double*) );		// allocates memory for the dataset
	for(i=0;i<n;i++)						
		x[i]=calloc( d,sizeof(double) );							
	readData( x,n,d,datafile );
  
  	/* ************** BUILD THE BVPTREE *************************/
  
	printf("**** building the bregman vantage point tree ****... \t");
	if (isHrCounter)
		isStart = QueryPerformanceCounter( &startCount );  
	root = buildBvpTree( x,n,d,bucketSize );		/* recursively builds the bbtree++*/
  
	if (isHrCounter)
		isEnd = QueryPerformanceCounter( &endCount );
	if (isStart && isEnd)
		printf( "done in %6.6f seconds \n",timediff( startCount,endCount,freqCount ) );
     
	//saves the bbtree:
	if(isTreefile)
		writeTree(root,d,treefile);
		
	printf("**** vantage point tree statistics *****\n");
	printf("\t tree depth = %d\n",maxDepth);
	printf("\t tree average depth = %6.2f\n",sumLeafDepth/(double)n);
	printf("\t number of interior nodes = %d\n",numNonLeafNodes);
	printf("\t number of leaves = %d\n",numLeafNodes);
	printf("\t average number of points per leaf = %6.2f\n",(double)n/(double)numLeafNodes);
	printf("\t construction cost = %d\n",constructionCost);

	//saves results of the experiment
	if(isResFile)
	//	writeResults(resultsFile,bucketSize,maxDepth,sumLeafDepth/(double)n,NumChildrenOfNonLeaves/(double)numNonLeafNodes,numNonLeafNodes,numLeafNodes,timediff( startCount,endCount,freqCount ));
		writeResults ( resultsFile,7,(double)maxDepth,sumLeafDepth/(double)n,(double)numNonLeafNodes,(double)numLeafNodes,(double)n/(double)numLeafNodes,(double)constructionCost,timediff( startCount,endCount,freqCount ) );	// add construction cost and time

	// free the memory
	for(i=0;i<n;i++)
		free(x[i]);
	free(x);
  
	deleteBvpTree(root);

	system("pause");
	return 0;
}

void processArgs(int argc, char**argv){
	int i=1;
	if(argc <= 1){
		printf("usage:\n buildbvpt -f dataFile -n numPoints -d dimPoints -t treeFile -r resultsFile [-s bucketsize] \n");
		exit(0);
	}
  
	while(i<argc){
		if(!strcmp(argv[i], "-d"))
			d = atoi(argv[++i]);
		else if(!strcmp(argv[i], "-n"))
			n=atoi(argv[++i]);
		else if(!strcmp(argv[i], "-f"))
			datafile = argv[++i];
		else if(!strcmp(argv[i], "-s"))
			bucketSize=atoi(argv[++i]);
		else if(!strcmp(argv[i], "-t")){
			treefile= argv[++i];
			isTreefile=1;
		}
		else if(!strcmp(argv[i], "-r")){
			resultsFile= argv[++i];
			isResFile=1;
		}
		else{
			fprintf(stderr,"unrecognized option.. exiting \n");
			exit(1);
		}
		i++;
	}
  
	if(n==-1 || d==-1 || datafile==NULL || treefile==NULL || resultsFile==NULL){
		fprintf(stderr,"more arguments needed.. exiting \n");
		exit(1);
	}
}  